﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace S3Browser
{
    public partial class FormBrowse : Form
    {
        SprightlySoftAWS.REST MyREST;
        SprightlySoftAWS.S3.ListBucket MyListBucket;
        SprightlySoftAWS.S3.ListBucket PopulateDeleteQueueListBucket;

        System.ComponentModel.BackgroundWorker ListBucketsBackgroundWorker;
        System.ComponentModel.BackgroundWorker ListBucketBackgroundWorker;
        System.ComponentModel.BackgroundWorker PopulateDeleteQueueBackgroundWorker;
        System.ComponentModel.BackgroundWorker DeleteItemBackgroundWorker;

        int ListViewAmazonSortColumn = 0;
        String Marker;
        ImageList MySmallImageList;
        System.Collections.ArrayList DeleteQueueArrayList;
        Boolean DeleteQueueKeepGoing;

        public String CurrentBucketName;
        public String CurrentPath;
        public String Delimiter;
        
        public FormBrowse()
        {
            InitializeComponent();
        }

        private void FormBrowse_Load(object sender, EventArgs e)
        {
            MyREST = new SprightlySoftAWS.REST();

            MyListBucket = new SprightlySoftAWS.S3.ListBucket();
            MyListBucket.ProgressChangedEvent += MyListBucket_ProgressChangedEvent;

            PopulateDeleteQueueListBucket = new SprightlySoftAWS.S3.ListBucket();

            ListBucketsBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            ListBucketsBackgroundWorker.DoWork += ListBucketsBackgroundWorker_DoWork;
            ListBucketsBackgroundWorker.RunWorkerCompleted += ListBucketsBackgroundWorker_RunWorkerCompleted;

            ListBucketBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            ListBucketBackgroundWorker.DoWork += ListBucketBackgroundWorker_DoWork;
            ListBucketBackgroundWorker.RunWorkerCompleted += ListBucketBackgroundWorker_RunWorkerCompleted;

            PopulateDeleteQueueBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            PopulateDeleteQueueBackgroundWorker.DoWork += PopulateDeleteQueueBackgroundWorker_DoWork;
            PopulateDeleteQueueBackgroundWorker.RunWorkerCompleted += PopulateDeleteQueueBackgroundWorker_RunWorkerCompleted;

            DeleteItemBackgroundWorker = new System.ComponentModel.BackgroundWorker();
            DeleteItemBackgroundWorker.DoWork += DeleteItemBackgroundWorker_DoWork;
            DeleteItemBackgroundWorker.RunWorkerCompleted += DeleteItemBackgroundWorker_RunWorkerCompleted;

            this.ResizeEnd += new System.EventHandler(FormBrowse_ResizeEnd);

            ListViewAmazon.DoubleClick += new System.EventHandler(ListViewAmazon_DoubleClick);
            ListViewAmazon.KeyDown += new System.Windows.Forms.KeyEventHandler(ListViewAmazon_KeyDown);
            ListViewAmazon.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(ListViewAmazon_ColumnClick);

            ToolStripStatusLabelAction.Text = "";
            ToolStripStatusLabelSummary.Text = "";

            ListViewAmazon.Sorting = SortOrder.Ascending;

            ListPath();
        }

        private void ListViewAmazon_ColumnClick(object sender, ColumnClickEventArgs e)
        {
            //Sorting ListView Items: http://msdn.microsoft.com/en-us/library/ms996467.aspx

            ToolStripStatusLabelAction.Text = "Sorting....";
            Application.DoEvents();

            if (e.Column != ListViewAmazonSortColumn)
            {
                ListViewAmazonSortColumn = e.Column;
                ListViewAmazon.Sorting = SortOrder.Ascending;
            }
            else
            {
                if (ListViewAmazon.Sorting == SortOrder.Ascending)
                {
                    ListViewAmazon.Sorting = SortOrder.Descending;
                }
                else
                {
                    ListViewAmazon.Sorting = SortOrder.Ascending;
                }
            }

            ListViewAmazon.Sort();
            ListViewAmazon.ListViewItemSorter = new ListViewItemComparer(e.Column, ListViewAmazon.Sorting, CurrentBucketName);

            ToolStripStatusLabelAction.Text = "";

        }

        private void ListViewAmazon_DoubleClick(object sender, EventArgs e)
        {
            ItemObject MyItemObject = (ItemObject)ListViewAmazon.SelectedItems[0].Tag;

            if (MyItemObject.IsBucket == true)
            {
                //a bucket was clicked
                CurrentBucketName = MyItemObject.BucketName;
                CurrentPath = "";
                ListPath();
            }
            else if (MyItemObject.IsFolder == true)
            {
                //a folder was clicked
                CurrentPath = MyItemObject.KeyName;
                ListPath();
            }
        }

        private void ToolStripButtonGoUp_Click(object sender, EventArgs e)
        {
            if (Delimiter != "" && CurrentPath.Contains(Delimiter) == true)
            {
                //remove the last folder from the path
                CurrentPath = CurrentPath.Substring(0, CurrentPath.Length - Delimiter.Length);
                if (CurrentPath.Contains(Delimiter) == true)
                {
                    CurrentPath = CurrentPath.Substring(0, CurrentPath.LastIndexOf(Delimiter) + 1);
                }
                else
                {
                    CurrentPath = "";
                }
                ListPath();
            }
            else if (CurrentPath == "")
            {
                //if there is no path, list all buckets
                CurrentBucketName = "";
                ListPath();
            }
        }

        private void ToolStripButtonRefresh_Click(object sender, EventArgs e)
        {
            ListPath();
        }

        private void ListPath()
        {

            //Amazon S3 Keys: http://docs.amazonwebservices.com/AmazonS3/latest/UsingKeys.html

            ToolStripTextBoxPath.Text = Delimiter + CurrentBucketName;
            if (CurrentPath != "")
            {
                ToolStripTextBoxPath.Text = ToolStripTextBoxPath.Text + Delimiter + CurrentPath;
            }

            if (CurrentBucketName == "")
            {
                //list all buckets
                ListBuckets();
            }
            else
            {
                //list the path
                ListBucket();
            }

        }

        private void EnableDisableStart()
        {
            //remove the item sorter so items are not sorted when added.  This will make listing large buckets quicker
            ListViewAmazon.ListViewItemSorter = null;
            ListViewAmazon.Clear();

            Properties.Settings MySettings = new Properties.Settings();

            if (CurrentBucketName == "")
            {
                //set the column headers for listing buckets
                ColumnHeader MyColumnHeader = new ColumnHeader();
                MyColumnHeader.Text = "Name";
                if (MySettings.BucketsCol0Width > 20)
                {
                    MyColumnHeader.Width = MySettings.BucketsCol0Width;
                }
                else
                {
                    MyColumnHeader.Width = 250;
                }
                ListViewAmazon.Columns.Add(MyColumnHeader);

                MyColumnHeader = new ColumnHeader();
                MyColumnHeader.Text = "Date Created";
                if (MySettings.BucketsCol1Width > 20)
                {
                    MyColumnHeader.Width = MySettings.BucketsCol1Width;
                }
                else
                {
                    MyColumnHeader.Width = 140;
                }
                ListViewAmazon.Columns.Add(MyColumnHeader);
            }
            else
            {
                //set the column headers for listing a bucket
                ColumnHeader MyColumnHeader = new ColumnHeader();
                MyColumnHeader.Text = "Name";
                if (MySettings.BucketCol0Width > 20)
                {
                    MyColumnHeader.Width = MySettings.BucketCol0Width;
                }
                else
                {
                    MyColumnHeader.Width = 250;
                }
                ListViewAmazon.Columns.Add(MyColumnHeader);

                MyColumnHeader = new ColumnHeader();
                MyColumnHeader.Text = "Size";
                if (MySettings.BucketCol1Width > 20)
                {
                    MyColumnHeader.Width = MySettings.BucketCol1Width;
                }
                else
                {
                    MyColumnHeader.Width = 75;
                }
                ListViewAmazon.Columns.Add(MyColumnHeader);

                MyColumnHeader = new ColumnHeader();
                MyColumnHeader.Text = "Last Modified";
                if (MySettings.BucketCol2Width > 20)
                {
                    MyColumnHeader.Width = MySettings.BucketCol2Width;
                }
                else
                {
                    MyColumnHeader.Width = 140;
                }
                ListViewAmazon.Columns.Add(MyColumnHeader);

                MyColumnHeader = new ColumnHeader();
                MyColumnHeader.Text = "Type";
                if (MySettings.BucketCol3Width > 20)
                {
                    MyColumnHeader.Width = MySettings.BucketCol3Width;
                }
                else
                {
                    MyColumnHeader.Width = 140;
                }
                ListViewAmazon.Columns.Add(MyColumnHeader);
            }

            //do not allow sorting by clicking a column header while listing is taking place
            ListViewAmazon.HeaderStyle = ColumnHeaderStyle.Nonclickable;

            ToolStripPath.Enabled = false;
            ToolStripButtons.Enabled = false;

        }

        private void EnableDisableEnd(Boolean SortListView)
        {
            if (SortListView == true)
            {
                ToolStripStatusLabelAction.Text = "Sorting....";
                Application.DoEvents();
                ListViewAmazon.Sort();
                ListViewAmazon.ListViewItemSorter = new ListViewItemComparer(ListViewAmazonSortColumn, ListViewAmazon.Sorting, CurrentBucketName);
            }

            ToolStripPath.Enabled = true;
            ToolStripButtons.Enabled = true;
            ListViewAmazon.HeaderStyle = ColumnHeaderStyle.Clickable;
            ToolStripStatusLabelAction.Text = "";
            ToolStripStatusLabelSummary.Text = ListViewAmazon.Items.Count.ToString("N0") + " items";
        }

        private void ListBuckets()
        {
            EnableDisableStart();

            //GET Service: http://docs.amazonwebservices.com/AmazonS3/latest/API/RESTServiceGET.html

            MySmallImageList = new ImageList();
            MySmallImageList.ColorDepth = ColorDepth.Depth16Bit;
            ListViewAmazon.SmallImageList = MySmallImageList;

            ToolStripStatusLabelAction.Text = "Listing....";
            ToolStripStatusLabelSummary.Text = "";

            Properties.Settings MySettings = new Properties.Settings();

            String RequestURL;
            RequestURL = MyREST.BuildS3RequestURL(true, "s3.amazonaws.com", "", "", "");

            String RequestMethod;
            RequestMethod = "GET";

            Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();
            ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

            String AuthorizationValue;
            AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
            ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

            System.Collections.Hashtable MyHashTable = new System.Collections.Hashtable();
            MyHashTable.Add("RequestURL", RequestURL);
            MyHashTable.Add("RequestMethod", RequestMethod);
            MyHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders);

            ListBucketsBackgroundWorker.RunWorkerAsync(MyHashTable);
        }

        private void ListBucketsBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            System.Collections.Hashtable MyHashTable = e.Argument as System.Collections.Hashtable;

            e.Result = MyREST.MakeRequest(MyHashTable["RequestURL"].ToString(), MyHashTable["RequestMethod"].ToString(), MyHashTable["ExtraRequestHeaders"] as Dictionary<String, String>, "");
        }

        private void ListBucketsBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            System.Diagnostics.Debug.Print("");
            System.Diagnostics.Debug.Print(MyREST.LogData);
            System.Diagnostics.Debug.Print("");

            //initalize the image list that will store icons for the list view items
            ImageList MySmallImageList = new ImageList();
            MySmallImageList.ColorDepth = ColorDepth.Depth16Bit;
            ListViewAmazon.SmallImageList = MySmallImageList;

            ItemObject MyItemObject;
            ListViewItem MyListViewItem;
            SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();
            Bitmap MyBitmap;

            String BucketName;
            DateTime BucketDate;
            String OwnerID;
            String OwnerName;

            System.Xml.XmlDocument XmlDoc;
            System.Xml.XmlNamespaceManager MyXmlNamespaceManager;
            System.Xml.XmlNode XmlNode;
            System.Xml.XmlNodeList XmlContentsNodeList;
            String ResponseMessage = "";

            //check if the result of the operation was successful
            if (Convert.ToBoolean(e.Result) == true)
            {
                ToolStripStatusLabelAction.Text = "Processing XML....";
                ListViewAmazon.BeginUpdate();

                //load the response XML from Amazon into an XmlDocument
                XmlDoc = new System.Xml.XmlDocument();
                XmlDoc.LoadXml(MyREST.ResponseString);

                //specify the namespace for XML path expressions
                MyXmlNamespaceManager = new System.Xml.XmlNamespaceManager(XmlDoc.NameTable);
                MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/");

                //extract all values from the response XML
                XmlNode = XmlDoc.SelectSingleNode("/amz:ListAllMyBucketsResult/amz:Owner/amz:ID", MyXmlNamespaceManager);
                OwnerID = XmlNode.InnerText;

                XmlNode = XmlDoc.SelectSingleNode("/amz:ListAllMyBucketsResult/amz:Owner/amz:DisplayName", MyXmlNamespaceManager);
                OwnerName = XmlNode.InnerText;

                XmlContentsNodeList = XmlDoc.SelectNodes("/amz:ListAllMyBucketsResult/amz:Buckets/amz:Bucket", MyXmlNamespaceManager);

                foreach (System.Xml.XmlNode XmlContentsNode in XmlContentsNodeList)
                {
                    XmlNode = XmlContentsNode.SelectSingleNode("./amz:Name", MyXmlNamespaceManager);
                    BucketName = XmlNode.InnerText;

                    XmlNode = XmlContentsNode.SelectSingleNode("./amz:CreationDate", MyXmlNamespaceManager);
                    BucketDate = Convert.ToDateTime(XmlNode.InnerText);

                    MyItemObject = new ItemObject();
                    MyItemObject.IsBucket = true;
                    MyItemObject.IsFolder = false;
                    MyItemObject.BucketName = BucketName;
                    MyItemObject.KeyName = "";
                    MyItemObject.DisplayName = BucketName;
                    MyItemObject.Size = -1;
                    MyItemObject.Date = BucketDate;
                    MyItemObject.FileType = "";
                    MyItemObject.ETag = "";
                    MyItemObject.OwnerID = OwnerID;
                    MyItemObject.OwnerName = OwnerName;
                    MyItemObject.SortName = BucketName;
                    MyItemObject.SortDate = BucketDate;
                    MyItemObject.SortFileType = "";

                    if (MySmallImageList.Images.ContainsKey("bucket") == false)
                    {
                        MyBitmap = MyHelper.GetIconByPath(System.IO.Path.GetTempPath(), true);  //use a folder icon for the bucket.  Call GetIconByPath and pass any folder name
                        MySmallImageList.Images.Add("bucket", MyBitmap);
                    }

                    MyListViewItem = new ListViewItem(BucketName);
                    MyListViewItem.SubItems.Add(BucketDate.ToString());
                    MyListViewItem.ImageKey = "bucket";
                    MyListViewItem.Tag = MyItemObject;
                    ListViewAmazon.Items.Add(MyListViewItem);
                }

                ListViewAmazon.EndUpdate();

                EnableDisableEnd(true);

            }
            else
            {
                if (MyREST.ResponseString == "")
                {
                    ResponseMessage = MyREST.ErrorDescription;
                }
                else
                {
                    XmlDoc = new System.Xml.XmlDocument();
                    XmlDoc.LoadXml(MyREST.ResponseString);

                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                    ResponseMessage = XmlNode.InnerText;
                }

                EnableDisableEnd(false);  //do not sort the list box after an error.  It may take a while if there are many items

                MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ListBucket()
        {
            EnableDisableStart();

            MySmallImageList = new ImageList();
            MySmallImageList.ColorDepth = ColorDepth.Depth16Bit;
            ListViewAmazon.SmallImageList = MySmallImageList;

            ToolStripStatusLabelAction.Text = "Listing....";
            ToolStripStatusLabelSummary.Text = "";

            Properties.Settings MySettings = new Properties.Settings();

            System.Collections.Hashtable MyHashTable = new System.Collections.Hashtable();
            MyHashTable.Add("UseSSL", true);
            MyHashTable.Add("RequestEndpoint", "s3.amazonaws.com");
            MyHashTable.Add("BucketName", CurrentBucketName);
            MyHashTable.Add("Delimiter", Delimiter);
            MyHashTable.Add("Prefix", CurrentPath);
            MyHashTable.Add("AWSAccessKeyId", MySettings.AWSAccessKeyId);
            MyHashTable.Add("AWSSecretAccessKey", MySettings.AWSSecretAccessKey);

            ListBucketBackgroundWorker.RunWorkerAsync(MyHashTable);
        }

        private void ListBucketBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            System.Collections.Hashtable MyHashTable = e.Argument as System.Collections.Hashtable;

            e.Result = MyListBucket.ListBucket(Convert.ToBoolean(MyHashTable["UseSSL"]), MyHashTable["RequestEndpoint"].ToString(), MyHashTable["BucketName"].ToString(), MyHashTable["Delimiter"].ToString(), MyHashTable["Prefix"].ToString(), MyHashTable["AWSAccessKeyId"].ToString(), MyHashTable["AWSSecretAccessKey"].ToString());
        }

        private void MyListBucket_ProgressChangedEvent()
        {
            //The ListBucket function list all items in a bucket.  Every time page is complete (1000 items)
            //the  ProgressChangedEvent is fired.  You can access the BucketItemsArrayList property
            //of the ListBucket class and add any new items to the ListView

            if (this.InvokeRequired == true)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { MyListBucket_ProgressChangedEvent(); }));
            }
            else
            {
                ItemObject MyItemObject;
                SprightlySoftAWS.S3.ListBucket.BucketItemObject MyBucketItemObject;
                ListViewItem MyListViewItem;
                SprightlySoftAWS.S3.Helper MyHelper = new SprightlySoftAWS.S3.Helper();

                Bitmap MyBitmap;
                String FileExtension = "";
                String FileType;

                ToolStripStatusLabelAction.Text = "Processing items....";
                ListViewAmazon.BeginUpdate();

                for (int i = ListViewAmazon.Items.Count; i <= MyListBucket.BucketItemsArrayList.Count - 1; i++)
                {
                    MyBucketItemObject = (SprightlySoftAWS.S3.ListBucket.BucketItemObject)MyListBucket.BucketItemsArrayList[i];

                    MyItemObject = new ItemObject();

                    if (MyBucketItemObject.IsFolder == true)
                    {
                        FileType = MyHelper.GetDisplayTypeForFolder();

                        MyItemObject.IsBucket = false;
                        MyItemObject.IsFolder = true;
                        MyItemObject.BucketName = MyBucketItemObject.BucketName;
                        MyItemObject.KeyName = MyBucketItemObject.KeyName;
                        MyItemObject.DisplayName = MyBucketItemObject.DisplayName;
                        MyItemObject.Size = MyBucketItemObject.Size;
                        MyItemObject.Date = MyBucketItemObject.LastModified;
                        MyItemObject.FileType = FileType;
                        MyItemObject.ETag = MyBucketItemObject.ETag;
                        MyItemObject.OwnerID = MyBucketItemObject.OwnerID;
                        MyItemObject.OwnerName = MyBucketItemObject.OwnerName;
                        MyItemObject.SortName = " " + MyBucketItemObject.DisplayName;  //prefix with a space so when ordered it's at the top
                        MyItemObject.SortDate = MyBucketItemObject.LastModified;
                        MyItemObject.SortFileType = " " + FileType;

                        if (MySmallImageList.Images.ContainsKey("folder") == false)
                        {
                            MyBitmap = MyHelper.GetIconByPath(System.IO.Path.GetTempPath(), true);  //use a folder icon for the bucket.  Call GetIconByPath and pass any folder name
                            MySmallImageList.Images.Add("folder", MyBitmap);
                        }

                        MyListViewItem = new ListViewItem(MyBucketItemObject.DisplayName);
                        MyListViewItem.SubItems.Add("");  //do not show the size for a folder
                        MyListViewItem.SubItems.Add("");  //do not show the date for a folder
                        MyListViewItem.SubItems.Add(FileType);
                        MyListViewItem.ImageKey = "folder";
                        MyListViewItem.Tag = MyItemObject;
                        ListViewAmazon.Items.Add(MyListViewItem);
                    }
                    else
                    {

                        try
                        {
                            FileType = MyHelper.GetDisplayTypeForFile(System.IO.Path.GetExtension(MyBucketItemObject.DisplayName));
                        }
                        catch
                        {
                            FileType = "";
                        }

                        MyItemObject.IsBucket = false;
                        MyItemObject.IsFolder = MyBucketItemObject.IsFolder;
                        MyItemObject.BucketName = MyBucketItemObject.BucketName;
                        MyItemObject.KeyName = MyBucketItemObject.KeyName;
                        MyItemObject.DisplayName = MyBucketItemObject.DisplayName;
                        MyItemObject.Size = MyBucketItemObject.Size;
                        MyItemObject.Date = MyBucketItemObject.LastModified;
                        MyItemObject.FileType = FileType;
                        MyItemObject.ETag = MyBucketItemObject.ETag;
                        MyItemObject.OwnerID = MyBucketItemObject.OwnerID;
                        MyItemObject.OwnerName = MyBucketItemObject.OwnerName;
                        MyItemObject.SortName = MyBucketItemObject.DisplayName;
                        MyItemObject.SortDate = MyBucketItemObject.LastModified;
                        MyItemObject.SortFileType = FileType;

                        try
                        {
                            FileExtension = System.IO.Path.GetExtension(MyBucketItemObject.DisplayName).ToLower();
                        }
                        catch
                        {
                            FileExtension = "";
                        }

                        if (FileExtension == "")
                        {
                            //a value is required for the FileExtension.  Hard code a value if none exists.
                            FileExtension = ".noextension";
                        }
                        else if (FileExtension.Length > 5)
                        {
                            //use the default file icon if the extension name is too long.  This will prevent the image list form being filled with too many icons for nonsense extensions
                            FileExtension = ".noextension";
                        }

                        //only add the icon to the ImageList is it does not exist in the ImageList
                        if (MySmallImageList.Images.ContainsKey(FileExtension) == false)
                        {
                            MyBitmap = MyHelper.GetIconByExtension(FileExtension, true);
                            if (MyBitmap != null)
                            {
                                MySmallImageList.Images.Add(FileExtension, MyBitmap);
                            }
                        }

                        MyListViewItem = new ListViewItem(MyBucketItemObject.DisplayName);
                        MyListViewItem.SubItems.Add(MyHelper.FormatByteSize(MyBucketItemObject.Size));
                        MyListViewItem.SubItems.Add(MyBucketItemObject.LastModified.ToString());
                        MyListViewItem.SubItems.Add(FileType);
                        MyListViewItem.ImageKey = FileExtension;
                        MyListViewItem.Tag = MyItemObject;
                        ListViewAmazon.Items.Add(MyListViewItem);
                    }
                }

                ListViewAmazon.EndUpdate();

                ToolStripStatusLabelAction.Text = "Listing....";
                ToolStripStatusLabelSummary.Text = ListViewAmazon.Items.Count.ToString("N0") + " items";
            }
        }

        private void ListBucketBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            System.Diagnostics.Debug.Print("");
            System.Diagnostics.Debug.Print(MyListBucket.LogData);
            System.Diagnostics.Debug.Print("");

            if (Convert.ToBoolean(e.Result) == true)
            {
                EnableDisableEnd(true);
            }
            else
            {
                String ResponseMessage = "";
                System.Xml.XmlDocument XmlDoc;
                System.Xml.XmlNode XmlNode;

                if (MyListBucket.ResponseString == "")
                {
                    ResponseMessage = MyListBucket.ErrorDescription;
                }
                else
                {
                    XmlDoc = new System.Xml.XmlDocument();
                    XmlDoc.LoadXml(MyListBucket.ResponseString);

                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                    ResponseMessage = XmlNode.InnerText;
                }

                EnableDisableEnd(false);  //do not sort the list box after an error.  It may take a while if there are many items

                MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ListViewAmazon_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
        {
            if (e.KeyCode == Keys.A && Control.ModifierKeys == Keys.Control)  //Ctrl-A  select all items
            {
                foreach (ListViewItem MyListViewItem in ListViewAmazon.Items)
                {
                    MyListViewItem.Selected = true;
                }
            }
            else if (e.KeyCode == Keys.F5)  //refresh the current path
            {
                ListPath();
            }
            else if (e.KeyCode == Keys.Delete)  //delete the selected items
            {
                DeleteItems();
            }
            else if (e.KeyCode == Keys.Escape)  //abort the listing of a bucket or buckets
            {
                if (MyREST.State == 1)
                {
                    MyREST.Abort();
                }
                else if (MyListBucket.State == 1)
                {
                    MyListBucket.Abort();
                }
            }
            else if (e.KeyCode == Keys.Enter)  //enter the selected bucket or folder
            {
                if (ListViewAmazon.SelectedItems.Count == 1)
                {
                    ItemObject MyItemObject = (ItemObject)ListViewAmazon.SelectedItems[0].Tag;

                    if (MyItemObject.IsBucket == true)
                    {
                        CurrentBucketName = MyItemObject.BucketName;
                        CurrentPath = "";
                        ListPath();
                    }
                    else if (MyItemObject.IsFolder == true)
                    {
                        CurrentPath = MyItemObject.KeyName;
                        ListPath();
                    }
                }
            }
            else if (e.KeyCode == Keys.Back)  //go up one level
            {
                if (Delimiter != "" && CurrentPath.Contains(Delimiter) == true)
                {
                    //remove the last folder from the path
                    CurrentPath = CurrentPath.Substring(0, CurrentPath.Length - Delimiter.Length);
                    if (CurrentPath.Contains(Delimiter) == true)
                    {
                        CurrentPath = CurrentPath.Substring(0, CurrentPath.LastIndexOf(Delimiter) + 1);
                    }
                    else
                    {
                        CurrentPath = "";
                    }
                    ListPath();
                }
                else if (CurrentPath == "")
                {
                    //if there is no path, list all buckets
                    CurrentBucketName = "";
                    ListPath();
                }
            }
        }

        private void ListViewAmazon_ColumnWidthChanged(object sender, System.Windows.Forms.ColumnWidthChangedEventArgs e)
        {
            Properties.Settings MySettings = new Properties.Settings();
            
            if (CurrentBucketName == "")
            {
                if (ListViewAmazon.Columns.Count == 2)  //only save the width if all columns are displayed
                {
                    MySettings.BucketsCol0Width = ListViewAmazon.Columns[0].Width;
                    MySettings.BucketsCol1Width = ListViewAmazon.Columns[1].Width;
                    MySettings.Save();
                }
            }
            else
            {
                if (ListViewAmazon.Columns.Count == 4)  //only save the width if all columns are displayed
                {
                    MySettings.BucketCol0Width = ListViewAmazon.Columns[0].Width;
                    MySettings.BucketCol1Width = ListViewAmazon.Columns[1].Width;
                    MySettings.BucketCol2Width = ListViewAmazon.Columns[2].Width;
                    MySettings.BucketCol3Width = ListViewAmazon.Columns[3].Width;
                    MySettings.Save();
                }
            }
        }

        private void FormBrowse_ResizeEnd(object sender, System.EventArgs e)
        {
            Properties.Settings MySettings = new Properties.Settings();
            MySettings.FormBrowseHeight = this.Height;
            MySettings.FormBrowseWidth = this.Width;
            MySettings.Save();
        }

        private void ToolStripButtonNewFolder_Click(object sender, EventArgs e)
        {
            if (CurrentBucketName == "")
            {
                //make sure the user is in a bucket
                MessageBox.Show("A folder can only be created in a bucket.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else if (Delimiter == "")
            {
                //a folder can only be created in folder view mode
                MessageBox.Show("A folder can only be created when a delimiter (folder view) is being used.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                //get the new folder name
                String NewFolderName;
                NewFolderName = Microsoft.VisualBasic.Interaction.InputBox("Enter the name of the new folder.", "New Folder", "", this.Location.X + 100, this.Location.Y + 150);

                String NewKeyName;
                NewKeyName = CurrentPath + NewFolderName + Delimiter;

                Properties.Settings MySettings = new Properties.Settings();

                if (NewFolderName == "")
                {
                    //the user did not enter a folder name, do nothing
                }
                else if (NewFolderName.Contains(Delimiter) == true)
                {
                    MessageBox.Show("The folder name cannot contain the delimiter character (" + Delimiter + ").", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    //check if the folder already exists
                    String RequestURL;
                    RequestURL = MyREST.BuildS3RequestURL(true, "s3.amazonaws.com", CurrentBucketName, NewKeyName, "");

                    String RequestMethod;
                    RequestMethod = "HEAD";

                    Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();
                    ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                    String AuthorizationValue;
                    AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                    ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                    Boolean RetBool;
                    RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, "");

                    if (MyREST.ResponseStatusCode == 404)
                    {
                        String PutXML;
                        PutXML = "";  //creating a folder is like uploading a file.  The folder object will be an empty file

                        RequestURL = MyREST.BuildS3RequestURL(true, "s3.amazonaws.com", CurrentBucketName, NewKeyName, "");

                        RequestMethod = "PUT";

                        ExtraRequestHeaders = new Dictionary<String, String>();
                        ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                        AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                        ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                        RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, PutXML);

                        if (RetBool == true)
                        {
                            //the folder was created.  Relist the path to see the new folder
                            ListPath();
                        }
                        else
                        {
                            String ResponseMessage;

                            if (MyREST.ResponseString == "")
                            {
                                ResponseMessage = MyREST.ErrorDescription;
                            }
                            else
                            {
                                System.Xml.XmlDocument XmlDoc;
                                System.Xml.XmlNode XmlNode;

                                XmlDoc = new System.Xml.XmlDocument();
                                XmlDoc.LoadXml(MyREST.ResponseString);

                                XmlNode = XmlDoc.SelectSingleNode("/Error/Message"); ;

                                ResponseMessage = XmlNode.InnerText;
                            }

                            MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                    }
                    else
                    {
                        MessageBox.Show("The folder already exists.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }

                }
            }
        }

        private void ToolStripButtonDelete_Click(object sender, EventArgs e)
        {
            DeleteItems();
        }

        private void DeleteItems()
        {
            if (ListViewAmazon.SelectedItems.Count > 0)
            {
                System.Windows.Forms.DialogResult MyDialogResult;
                if (ListViewAmazon.SelectedItems.Count == 1)
                {
                    ItemObject MyItemObject = ListViewAmazon.SelectedItems[0].Tag as ItemObject;
                    MyDialogResult = MessageBox.Show("Are you sure you would like to permanently delete " + MyItemObject.DisplayName + "?", "Delete Item", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                }
                else
                {
                    MyDialogResult = MessageBox.Show("Are you sure you would like to permanently delete the " + ListViewAmazon.SelectedItems.Count + " selected items?", "Delete Items", MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                }

                if (MyDialogResult == System.Windows.Forms.DialogResult.Yes)
                {
                    System.Collections.ArrayList DeleteQueueArrayList = new System.Collections.ArrayList();
                    DeleteQueueKeepGoing = true;

                    SprightlySoftAWS.S3.ListBucket.BucketItemObject MyBucketItemObject;
                    ItemObject MyItemObject;

                    //add each selected item to the DeleteQueueArrayList
                    foreach (ListViewItem MyListViewItem in ListViewAmazon.SelectedItems)
                    {
                        //convert the Tag to a BucketItemObject and add it to the ArrayList
                        MyItemObject = MyListViewItem.Tag as ItemObject;

                        MyBucketItemObject = new SprightlySoftAWS.S3.ListBucket.BucketItemObject();

                        MyBucketItemObject.BucketName = MyItemObject.BucketName;
                        MyBucketItemObject.DisplayName = MyItemObject.DisplayName;
                        MyBucketItemObject.ETag = MyItemObject.ETag;
                        MyBucketItemObject.IsFolder = MyItemObject.IsFolder;
                        MyBucketItemObject.KeyName = MyItemObject.KeyName;
                        MyBucketItemObject.LastModified = MyItemObject.Date;
                        MyBucketItemObject.OwnerID = MyItemObject.OwnerID;
                        MyBucketItemObject.OwnerName = MyItemObject.OwnerName;
                        MyBucketItemObject.Size = MyItemObject.Size;

                        DeleteQueueArrayList.Add(MyBucketItemObject);
                    }

                    ToolStripStatusLabelAction.Text = "Getting items to delete " + DeleteQueueArrayList.Count.ToString("N0") + "....";
                    ListViewAmazon.Enabled = false;
                    ToolStripButtonGoUp.Enabled = false;
                    ToolStripButtonRefresh.Enabled = false;
                    ToolStripButtonDelete.Enabled = false;
                    ToolStripButtonNewFolder.Enabled = false;
                    ToolStripButtonAbort.Visible = true;

                    Properties.Settings MySettings = new Properties.Settings();

                    //In a background worker go through the ArrayList and add each item in each folder to the ArrayList.
                    //Amazon does not have a feature to recursively delete a folder.  Each item must be deleted individually.
                    System.Collections.Hashtable MyHashTable = new System.Collections.Hashtable();
                    MyHashTable.Add("UseSSL", true);
                    MyHashTable.Add("RequestEndpoint", "s3.amazonaws.com");
                    MyHashTable.Add("AWSAccessKeyId", MySettings.AWSAccessKeyId);
                    MyHashTable.Add("AWSSecretAccessKey", MySettings.AWSSecretAccessKey);
                    MyHashTable.Add("DeleteQueueArrayList", DeleteQueueArrayList);

                    PopulateDeleteQueueBackgroundWorker.RunWorkerAsync(MyHashTable);
                }
            }
        }

        private void PopulateDeleteQueueBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            System.Collections.Hashtable MyArgumentHashTable = e.Argument as System.Collections.Hashtable;

            System.Collections.ArrayList MyArrayList;
            MyArrayList = MyArgumentHashTable["DeleteQueueArrayList"] as System.Collections.ArrayList;

            Boolean RetBool = true;
            SprightlySoftAWS.S3.ListBucket.BucketItemObject MyArrayListBucketItemObject;

            for (int i = 0; i <= MyArrayList.Count - 1; i++)
            {
                MyArrayListBucketItemObject = MyArrayList[i] as SprightlySoftAWS.S3.ListBucket.BucketItemObject;

                //if the item is a folder, add each item in the folder to the delete queue
                if (MyArrayListBucketItemObject.IsFolder == true)
                {
                    //call ListBucket with no delimiter to get a recursive list of all items in the folder
                    RetBool = PopulateDeleteQueueListBucket.ListBucket(Convert.ToBoolean(MyArgumentHashTable["UseSSL"]), MyArgumentHashTable["RequestEndpoint"].ToString(), MyArrayListBucketItemObject.BucketName, "", MyArrayListBucketItemObject.KeyName, MyArgumentHashTable["AWSAccessKeyId"].ToString(), MyArgumentHashTable["AWSSecretAccessKey"].ToString());

                    if (RetBool == true)
                    {
                        foreach (SprightlySoftAWS.S3.ListBucket.BucketItemObject MyBucketItemObject in PopulateDeleteQueueListBucket.BucketItemsArrayList)
                        {
                            MyArrayList.Add(MyBucketItemObject);
                        }
                    }
                    else
                    {
                        //if an error occurs, exit
                        break;
                    }
                }

                if (DeleteQueueKeepGoing == false)
                {
                    break;
                }
            }

            System.Collections.Hashtable MyHashTable = new System.Collections.Hashtable();

            if (DeleteQueueKeepGoing == false)
            {
                MyHashTable.Add("RetBool", false);
                MyHashTable.Add("ResponseXML", "");
                MyHashTable.Add("ErrorNumber", 1172);
                MyHashTable.Add("ErrorDescription", "The operation was aborted.");
                MyHashTable.Add("DeleteQueueArrayList", MyArrayList);
            }
            else
            {
                MyHashTable.Add("RetBool", RetBool);
                MyHashTable.Add("ResponseXML", PopulateDeleteQueueListBucket.ResponseString);
                MyHashTable.Add("ErrorNumber", PopulateDeleteQueueListBucket.ErrorNumber);
                MyHashTable.Add("ErrorDescription", PopulateDeleteQueueListBucket.ErrorDescription);
                MyHashTable.Add("DeleteQueueArrayList", MyArrayList);
            }

            e.Result = MyHashTable;
        }

        private void PopulateDeleteQueueListBucket_ProgressChangedEvent()
        {
            //update the status bar while listing a folder that has thousands of items
            if (this.InvokeRequired == true)
            {
                this.BeginInvoke(new MethodInvoker(delegate() { PopulateDeleteQueueListBucket_ProgressChangedEvent(); }));
            }
            else
            {
                ToolStripStatusLabelAction.Text = "Getting items to delete " + (DeleteQueueArrayList.Count + PopulateDeleteQueueListBucket.BucketItemsArrayList.Count).ToString("N0") + "....";
            }
        }

        private void PopulateDeleteQueueBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            System.Collections.Hashtable MyResultHashTable = e.Result as System.Collections.Hashtable;

            if (Convert.ToBoolean(MyResultHashTable["RetBool"]) == true)
            {
                DeleteQueueArrayList = MyResultHashTable["DeleteQueueArrayList"] as System.Collections.ArrayList;

                SprightlySoftAWS.S3.ListBucket.BucketItemObject MyBucketItemObject = DeleteQueueArrayList[DeleteQueueArrayList.Count - 1] as SprightlySoftAWS.S3.ListBucket.BucketItemObject;

                ToolStripStatusLabelAction.Text = "Deleting " + MyBucketItemObject.KeyName + ", " + DeleteQueueArrayList.Count.ToString("N0") + " items remaining....";

                Properties.Settings MySettings = new Properties.Settings();

                String RequestURL;
                RequestURL = MyREST.BuildS3RequestURL(true, "s3.amazonaws.com", MyBucketItemObject.BucketName, MyBucketItemObject.KeyName, "");

                String RequestMethod;
                RequestMethod = "DELETE";

                Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();
                ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                String AuthorizationValue;
                AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                //delete the last item in the array list first so items in a folder are deleted before the folder itself
                System.Collections.Hashtable MyHashTable = new System.Collections.Hashtable();
                MyHashTable.Add("RequestURL", RequestURL);
                MyHashTable.Add("RequestMethod", RequestMethod);
                MyHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders);

                DeleteItemBackgroundWorker.RunWorkerAsync(MyHashTable);
            }
            else
            {
                ToolStripStatusLabelAction.Text = "";
                ListViewAmazon.Enabled = true;
                ToolStripButtonGoUp.Enabled = true;
                ToolStripButtonRefresh.Enabled = true;
                ToolStripButtonDelete.Enabled = true;
                ToolStripButtonNewFolder.Enabled = true;
                ToolStripButtonAbort.Visible = false;

                String ResponseMessage;
                System.Xml.XmlDocument XmlDoc;
                System.Xml.XmlNode XmlNode;

                if (MyListBucket.ResponseString == "")
                {
                    ResponseMessage = MyResultHashTable["ErrorDescription"].ToString();
                }
                else
                {
                    XmlDoc = new System.Xml.XmlDocument();
                    XmlDoc.LoadXml(MyResultHashTable["ResponseXML"].ToString());

                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                    ResponseMessage = XmlNode.InnerText;
                }

                MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void DeleteItemBackgroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            System.Collections.Hashtable MyHashTable = e.Argument as System.Collections.Hashtable;
            e.Result = MyREST.MakeRequest(MyHashTable["RequestURL"].ToString(), MyHashTable["RequestMethod"].ToString(), MyHashTable["ExtraRequestHeaders"] as Dictionary<String, String>, "");
        }

        private void DeleteItemBackgroundWorker_RunWorkerCompleted(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            System.Diagnostics.Debug.Print("");
            System.Diagnostics.Debug.Print(MyREST.LogData);
            System.Diagnostics.Debug.Print("");

            if (Convert.ToBoolean(e.Result) == true)
            {
                //remove the last item from the arraylist as it was just deleted
                DeleteQueueArrayList.RemoveAt(DeleteQueueArrayList.Count - 1);

                if (DeleteQueueKeepGoing == false)
                {
                    ToolStripStatusLabelAction.Text = "";
                    ListViewAmazon.Enabled = true;
                    ToolStripButtonGoUp.Enabled = true;
                    ToolStripButtonRefresh.Enabled = true;
                    ToolStripButtonDelete.Enabled = true;
                    ToolStripButtonNewFolder.Enabled = true;
                    ToolStripButtonAbort.Visible = false;

                    MessageBox.Show("The operation was aborted.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                    ListPath();
                }
                else if (DeleteQueueArrayList.Count > 0)
                {
                    SprightlySoftAWS.S3.ListBucket.BucketItemObject MyBucketItemObject = DeleteQueueArrayList[DeleteQueueArrayList.Count - 1] as SprightlySoftAWS.S3.ListBucket.BucketItemObject;

                    ToolStripStatusLabelAction.Text = "Deleting " + MyBucketItemObject.KeyName + ", " + DeleteQueueArrayList.Count.ToString("N0") + " items remaining....";

                    Properties.Settings MySettings = new Properties.Settings();

                    String RequestURL;
                    RequestURL = MyREST.BuildS3RequestURL(true, "s3.amazonaws.com", MyBucketItemObject.BucketName, MyBucketItemObject.KeyName, "");

                    String RequestMethod;
                    RequestMethod = "DELETE";

                    Dictionary<String, String> ExtraRequestHeaders = new Dictionary<String, String>();
                    ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                    String AuthorizationValue;
                    AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, MySettings.AWSAccessKeyId, MySettings.AWSSecretAccessKey);
                    ExtraRequestHeaders.Add("Authorization", AuthorizationValue);

                    //delete the next item
                    System.Collections.Hashtable MyHashTable = new System.Collections.Hashtable();
                    MyHashTable.Add("RequestURL", RequestURL);
                    MyHashTable.Add("RequestMethod", RequestMethod);
                    MyHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders);

                    DeleteItemBackgroundWorker.RunWorkerAsync(MyHashTable);
                }
                else
                {
                    ToolStripStatusLabelAction.Text = "";
                    ListViewAmazon.Enabled = true;
                    ToolStripButtonGoUp.Enabled = true;
                    ToolStripButtonRefresh.Enabled = true;
                    ToolStripButtonDelete.Enabled = true;
                    ToolStripButtonNewFolder.Enabled = true;
                    ToolStripButtonAbort.Visible = false;

                    ListPath();
                }
            }
            else
            {
                ToolStripStatusLabelAction.Text = "";
                ListViewAmazon.Enabled = true;
                ToolStripButtonGoUp.Enabled = true;
                ToolStripButtonRefresh.Enabled = true;
                ToolStripButtonDelete.Enabled = true;
                ToolStripButtonNewFolder.Enabled = true;
                ToolStripButtonAbort.Visible = false;

                String ResponseMessage;
                System.Xml.XmlDocument XmlDoc;
                System.Xml.XmlNode XmlNode;

                if (MyREST.ResponseString == "")
                {
                    ResponseMessage = MyREST.ErrorDescription;
                }
                else
                {
                    XmlDoc = new System.Xml.XmlDocument();
                    XmlDoc.LoadXml(MyREST.ResponseString);

                    XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                    ResponseMessage = XmlNode.InnerText;
                }

                MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);

                ListPath();
            }
        }

        private void ToolStripButtonAbort_Click(object sender, EventArgs e)
        {
            //abort any operations that are in progress and set a global variable to indicate that deletion should not continue
            if (PopulateDeleteQueueListBucket.State == 1)
            {
                PopulateDeleteQueueListBucket.Abort();
            }
            else if (MyREST.State == 1)
            {
                MyREST.Abort();
            }
            DeleteQueueKeepGoing = false;
        }


    }
}
