﻿using System;
using System.Collections.Generic;
using System.Text;

namespace S3Upload
{
    class Program
    {
        private static SprightlySoftAWS.S3.UploadEstimate MyUpload;

        private static int ExitCode;
        private static int OutputLevel;
        private static Boolean ShowTransferProgress;
        private static DateTime ProgressUpdateShown;
        private static long MultipartPartSizeBytes;
        private static Boolean UseServerSideEncryption;
        private static int PartCounter = 0;
        private static long FileLength;

        static void Main(string[] args)
        {
            ExitCode = 0;

            AppDomain currentDomain = AppDomain.CurrentDomain;
            currentDomain.UnhandledException += new UnhandledExceptionEventHandler(MyUnhandledException);

            MyUpload = new SprightlySoftAWS.S3.UploadEstimate();
            System.Net.IWebProxy MyIWebProxy;
            MyIWebProxy = System.Net.WebRequest.DefaultWebProxy;
            MyIWebProxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
            MyUpload.Proxy = MyIWebProxy;

            MyUpload.ProgressChangedEvent += MyUpload_ProgressChangedEvent;
            MyUpload.StateChangedEvent += MyUpload_StateChangedEvent;

            String AWSAccessKeyId = GetValueFromConfigOrArgument(args, "AWSAccessKeyId");
            String AWSSecretAccessKey = GetValueFromConfigOrArgument(args, "AWSSecretAccessKey");

            String RequestEndpoint = GetValueFromConfigOrArgument(args, "RequestEndpoint");

            Boolean UseSSL;
            if (Boolean.TryParse(GetValueFromConfigOrArgument(args, "UseSSL"), out UseSSL) == false)
            {
                UseSSL = true;
            }

            String BucketName = GetValueFromConfigOrArgument(args, "BucketName");
            String KeyName = GetValueFromConfigOrArgument(args, "KeyName");
            String LocalFile = GetValueFromConfigOrArgument(args, "LocalFile");
            String UploadHeaders = GetValueFromConfigOrArgument(args, "UploadHeaders");

            Boolean SaveTimestampsInMetadata;
            if (Boolean.TryParse(GetValueFromConfigOrArgument(args, "SaveTimestampsInMetadata"), out SaveTimestampsInMetadata) == false)
            {
                SaveTimestampsInMetadata = false;
            }

            Boolean SaveETagInMetadataForMultipart;
            if (Boolean.TryParse(GetValueFromConfigOrArgument(args, "SaveETagInMetadataForMultipart"), out SaveETagInMetadataForMultipart) == false)
            {
                SaveETagInMetadataForMultipart = false;
            }

            int S3FolderExistsCheck;
            if (int.TryParse(GetValueFromConfigOrArgument(args, "S3FolderExistsCheck"), out S3FolderExistsCheck) == false)
            {
                S3FolderExistsCheck = 1;
            }

            Boolean UseMultipartUpload;
            if (Boolean.TryParse(GetValueFromConfigOrArgument(args, "UseMultipartUpload"), out UseMultipartUpload) == false)
            {
                UseMultipartUpload = true;
            }

            if (long.TryParse(GetValueFromConfigOrArgument(args, "MultipartPartSizeBytes"), out MultipartPartSizeBytes) == false)
            {
                MultipartPartSizeBytes = 5242880;
            }

            if (Boolean.TryParse(GetValueFromConfigOrArgument(args, "UseServerSideEncryption"), out UseServerSideEncryption) == false)
            {
                UseServerSideEncryption = false;
            }

            Single TransferSpeedLimitKBps;
            if (Single.TryParse(GetValueFromConfigOrArgument(args, "TransferSpeedLimitKBps"), out TransferSpeedLimitKBps) == false)
            {
                TransferSpeedLimitKBps = 0;
            }

            if (Boolean.TryParse(GetValueFromConfigOrArgument(args, "ShowTransferProgress"), out ShowTransferProgress) == false)
            {
                ShowTransferProgress = true;
            }
            
            int S3ErrorRetries;
            if (int.TryParse(GetValueFromConfigOrArgument(args, "S3ErrorRetries"), out S3ErrorRetries) == false)
            {
                S3ErrorRetries = 10;
            }

            if (int.TryParse(GetValueFromConfigOrArgument(args, "OutputLevel"), out OutputLevel) == false)
            {
                OutputLevel = 1;
            }

            
            WriteToOutput("Program started.   " + DateTime.Now, 2);

            WriteToOutput("Input parameters:", 2);
            WriteToOutput("   AWSAccessKeyId=" + AWSAccessKeyId, 2);
            WriteToOutput("   AWSSecretAccessKey=" + AWSSecretAccessKey, 2);
            WriteToOutput("   UseSSL=" + UseSSL, 2);
            WriteToOutput("   RequestEndpoint=" + RequestEndpoint, 2);
            WriteToOutput("   BucketName=" + BucketName, 2);
            WriteToOutput("   KeyName=" + KeyName, 2);
            WriteToOutput("   LocalFile=" + LocalFile, 2);
            WriteToOutput("   UploadHeaders=" + UploadHeaders, 2);
            WriteToOutput("   SaveTimestampsInMetadata=" + SaveTimestampsInMetadata, 2);
            WriteToOutput("   SaveETagInMetadataForMultipart=" + SaveETagInMetadataForMultipart, 2);
            WriteToOutput("   S3FolderExistsCheck=" + S3FolderExistsCheck, 2);
            WriteToOutput("   UseMultipartUpload=" + UseMultipartUpload, 2);
            WriteToOutput("   MultipartPartSizeBytes=" + MultipartPartSizeBytes, 2);
            WriteToOutput("   UseServerSideEncryption=" + UseServerSideEncryption, 2);
            WriteToOutput("   TransferSpeedLimitKBps=" + TransferSpeedLimitKBps, 2);
            WriteToOutput("   ShowTransferProgress=" + ShowTransferProgress, 2);
            WriteToOutput("   S3ErrorRetries=" + S3ErrorRetries, 2);
            WriteToOutput("   OutputLevel=" + OutputLevel, 2);


            //Check that the input values are valid.
            if (AWSAccessKeyId == "")
            {
                WriteToOutput("Error, the AWSAccessKeyId is empty.  Set it in the .config file or pass it in the command line.", 1, true);
                ExitCode = -1;
            }
            else if (AWSSecretAccessKey == "")
            {
                WriteToOutput("Error, the AWSSecretAccessKey is empty.  Set it in the .config file or pass it in the command line.", 1, true);
                ExitCode = -1;
            }
            else if (BucketName == "")
            {
                WriteToOutput("Error, the BucketName is empty.  Set it in the .config file or pass it in the command line.", 1, true);
                ExitCode = -1;
            }
            else if (KeyName == "")
            {
                WriteToOutput("Error, the KeyName is empty.  Set it in the .config file or pass it in the command line.", 1, true);
                ExitCode = -1;
            }
            else if (LocalFile == "")
            {
                WriteToOutput("Error, the LocalFile is empty.  Set it in the .config file or pass it in the command line.", 1, true);
                ExitCode = -1;
            }
            else if (System.IO.File.Exists(LocalFile) == false)
            {
                WriteToOutput("Error, the LocalFile does not exist.  Specify a valid path.  LocalFile=" + LocalFile, 1, true);
                ExitCode = -1;
            }
            else if (MultipartPartSizeBytes < 5242880)
            {
                WriteToOutput("Error, the MultipartPartSizeBytes is too small.  The minimum MultipartPartSizeBytes is 5242880.", 1, true);
                ExitCode = -1;
            }


            SprightlySoftAWS.REST MyREST = new SprightlySoftAWS.REST();
            String RequestURL;
            String RequestMethod;
            Boolean RetBool;
            String ResponseErrorMessage = "";

            int ErrorNumber = 0;
            String ErrorDescription = "";
            String LogData = "";
            System.Collections.Generic.Dictionary<String, String> RequestHeaders = new System.Collections.Generic.Dictionary<String, String>();
            int ResponseStatusCode = 0;
            String ResponseStatusDescription = "";
            System.Collections.Generic.Dictionary<String, String> ResponseHeaders = new System.Collections.Generic.Dictionary<String, String>();
            String ResponseString = "";

            if (ExitCode == 0)
            {
                //Check if the account info is valid.
                //Get the bucket location.

                //GET Bucket location: http://docs.amazonwebservices.com/AmazonS3/latest/API/RESTBucketLocationGET.html
                //Regions and Endpoints:  http://docs.amazonwebservices.com/general/latest/gr/index.html?rande.html

                RequestURL = MyREST.BuildS3RequestURL(UseSSL, "s3.amazonaws.com", BucketName, "", "?location");
                RequestMethod = "GET";

                RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, null, "", S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                if (RetBool == true)
                {
                    if (RequestEndpoint == "")
                    {
                        System.Xml.XmlDocument XmlDoc;
                        System.Xml.XmlNamespaceManager MyXmlNamespaceManager;
                        System.Xml.XmlNode XmlNode;

                        XmlDoc = new System.Xml.XmlDocument();
                        XmlDoc.LoadXml(ResponseString);

                        MyXmlNamespaceManager = new System.Xml.XmlNamespaceManager(XmlDoc.NameTable);
                        MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/");

                        XmlNode = XmlDoc.SelectSingleNode("/amz:LocationConstraint", MyXmlNamespaceManager);

                        if (XmlNode.InnerText == "")
                        {
                            RequestEndpoint = "s3.amazonaws.com";
                        }
                        else if (XmlNode.InnerText == "us-west-1")
                        {
                            RequestEndpoint = "s3-us-west-1.amazonaws.com";
                        }
                        else if (XmlNode.InnerText == "EU")
                        {
                            RequestEndpoint = "s3-eu-west-1.amazonaws.com";
                        }
                        else if (XmlNode.InnerText == "ap-southeast-1")
                        {
                            RequestEndpoint = "s3-ap-southeast-1.amazonaws.com";
                        }
                        else if (XmlNode.InnerText == "ap-northeast-1")
                        {
                            RequestEndpoint = "s3-ap-northeast-1.amazonaws.com";
                        }
                        else
                        {
                            WriteToOutput("Error, the RequestEndpoint could not be determined.  LocationConstraint=" + XmlNode.InnerText + "  Set RequestEndpoint in the .config file or pass it in the command line.  See http://docs.amazonwebservices.com/general/latest/gr/rande.html#s3_region for a list of endpoints.", 1, true);
                            ExitCode = -1;
                        }

                        if (RequestEndpoint != "")
                        {
                            WriteToOutput("RequestEndpoint=" + RequestEndpoint, 2);
                        }
                    }
                }
                else
                {
                    WriteToOutput(LogData, 2);
                    ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                    WriteToOutput(ResponseErrorMessage, 1, true);
                    ExitCode = -1;
                }
            }


            if (ExitCode == 0)
            {
                if (S3FolderExistsCheck > 0 && KeyName.Contains("/") == true)
                {
                    //Check if the S3 folders exists.
                    String TempKeyName;
                    int StartIndex = -1;
                    int IndexValue;

                    do
                    {
                        IndexValue = KeyName.IndexOf("/", StartIndex + 1);

                        if (IndexValue > -1)
                        {
                            TempKeyName = KeyName.Substring(0, IndexValue + 1);
                            StartIndex = IndexValue;

                            RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, TempKeyName, "");
                            RequestMethod = "HEAD";

                            RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, null, "", S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                            if (RetBool == true)
                            {
                                //The folder exists.
                            }
                            else if (ResponseStatusCode == 404)
                            {
                                //The folder does not exists.

                                if (S3FolderExistsCheck == 1)
                                {
                                    //Show the user an error.
                                    WriteToOutput("Error, S3 folder does not exist.  Folder KeyName=" + TempKeyName, 1, true);
                                    ExitCode = -1;
                                    break;
                                }
                                else if (S3FolderExistsCheck == 2)
                                {
                                    //Create the folder.
                                    RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, TempKeyName, "");
                                    RequestMethod = "PUT";

                                    RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, null, "", S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                                    if (RetBool == true)
                                    {
                                        //Create S3 folder successful.
                                        WriteToOutput("S3 folder created.  Folder KeyName=" + TempKeyName);
                                    }
                                    else
                                    {
                                        WriteToOutput(LogData, 2);
                                        ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                                        WriteToOutput(ResponseErrorMessage, 1, true);
                                        ExitCode = -1;
                                        break;
                                    }
                                }
                            }
                            else
                            {
                                WriteToOutput(LogData, 2);
                                ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                                WriteToOutput(ResponseErrorMessage, 1, true);
                                ExitCode = -1;
                                break;
                            }
                        }

                    } while (IndexValue > 0);

                }
            }


            if (ExitCode == 0)
            {
                String LocalETag = "";
                SprightlySoftAWS.S3.CalculateHash MyCalculateHash = new SprightlySoftAWS.S3.CalculateHash();
                System.Collections.Generic.Dictionary<String, String> ExtraRequestHeaders = new System.Collections.Generic.Dictionary<String, String>();
                SprightlySoftAWS.S3.Helper MyS3Helper = new SprightlySoftAWS.S3.Helper();

                if (TransferSpeedLimitKBps > 0)
                {
                    MyUpload.LimitKBpsSpeed = TransferSpeedLimitKBps;
                }

                System.IO.FileInfo MyFileInfo;
                MyFileInfo = new System.IO.FileInfo(LocalFile);

                if (UseMultipartUpload == true && SaveETagInMetadataForMultipart == true && MyFileInfo.Length > MultipartPartSizeBytes)
                {
                    LocalETag = MyCalculateHash.CalculateETagFromFile(LocalFile);
                    ExtraRequestHeaders.Add("x-amz-meta-multipart-etag", LocalETag);
                }

                if (SaveTimestampsInMetadata == true)
                {
                    ExtraRequestHeaders.Add("x-amz-meta-local-date-modified", MyFileInfo.LastWriteTime.ToFileTimeUtc().ToString());
                    ExtraRequestHeaders.Add("x-amz-meta-local-date-created", MyFileInfo.CreationTime.ToFileTimeUtc().ToString());
                }

                if (UseServerSideEncryption == true)
                {
                    ExtraRequestHeaders.Add("x-amz-server-side-encryption", "AES256");
                }

                Dictionary<String, String> UploadHeadersDictionary;
                UploadHeadersDictionary = BuildDictionaryFromString(UploadHeaders);

                //Add the user supplied headers to the other headers that will be sent to S3.
                foreach (KeyValuePair<String, String> MyKeyValuePair in UploadHeadersDictionary)
                {
                    ExtraRequestHeaders.Add(MyKeyValuePair.Key, MyKeyValuePair.Value);
                }

                WriteToOutput("Uploading " + MyFileInfo.FullName);
                WriteToOutput("Size=" + MyS3Helper.FormatByteSize(MyFileInfo.Length), 2);

                RetBool = MultipartUpload(AWSAccessKeyId, AWSSecretAccessKey, UseSSL, RequestEndpoint, BucketName, KeyName, ExtraRequestHeaders, LocalFile, UseMultipartUpload, LocalETag, S3ErrorRetries, ref LogData, ref ResponseErrorMessage);

                if (RetBool == true)
                {
                    WriteToOutput("Upload successful.");
                }
                else
                {
                    WriteToOutput(LogData, 2);
                    WriteToOutput("Upload failed.", 1, true);
                    WriteToOutput(ResponseErrorMessage, 1, true);
                    ExitCode = -1;
                }
            }

            Environment.ExitCode = ExitCode;

            WriteToOutput("Program complete.   " + DateTime.Now, 2);
            WriteToOutput("ExitCode=" + ExitCode, 2);
        }


        private static Boolean MultipartUpload(String AWSAccessKeyId, String AWSSecretAccessKey, Boolean UseSSL, String RequestEndpoint, String BucketName, String KeyName, Dictionary<String, String> UserRequestHeaders, String LocalFile, Boolean UseMultipartUpload, String LocalETag, int S3ErrorRetries, ref String LogData, ref String ResponseErrorMessage)
        {
            Boolean ReturnValue = true;
            SprightlySoftAWS.REST MyREST = new SprightlySoftAWS.REST();
            SprightlySoftAWS.S3.Helper MyS3Helper = new SprightlySoftAWS.S3.Helper();
            SprightlySoftAWS.S3.CalculateHash MyCalculateHash = new SprightlySoftAWS.S3.CalculateHash();
            String LocalMD5;

            String RequestURL;
            Boolean RetBool;
            int ErrorNumber = 0;
            String ErrorDescription = "";
            Dictionary<String, String> RequestHeaders = new Dictionary<String, String>();
            int ResponseStatusCode = 0;
            String ResponseStatusDescription = "";
            Dictionary<String, String> ResponseHeaders = new Dictionary<String, String>();
            String ResponseString = "";

            String QueryString;
            String RequestMethod;
            System.Collections.Generic.Dictionary<String, String> ExtraRequestHeaders;
            String UploadId = "";

            System.Xml.XmlDocument MyXmlDocument;
            System.Xml.XmlNamespaceManager MyXmlNamespaceManager;
            System.Xml.XmlNode MyXmlNode;
            System.Xml.XmlNodeList MyXmlNodeList;

            LogData = "";
            PartCounter = 0;

            System.IO.FileInfo MyFileInfo;
            MyFileInfo = new System.IO.FileInfo(LocalFile);
            FileLength = MyFileInfo.Length;

            if (MyFileInfo.Length > MultipartPartSizeBytes && UseMultipartUpload == true)
            {
                //The file is big enough for multipart upload.

                //Validate that there will be less that 10,000 parts
                if (MyFileInfo.Length / MultipartPartSizeBytes > 10000)
                {
                    ResponseErrorMessage = "The number of parts will exceed 10,000.  Increase the MultipartPartSizeBytes to decrease the number of parts.  File size=" + MyFileInfo.Length + "  MultipartPartSizeBytes=" + MultipartPartSizeBytes;
                    ReturnValue = false;
                }

                if (ReturnValue == true)
                {
                    //List multipart uploads to see if the file is partially uploaded.
                    //List Multipart Uploads: http://docs.amazonwebservices.com/AmazonS3/latest/API/mpUploadListMPUpload.html

                    RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, "", "?uploads");
                    RequestMethod = "GET";

                    RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, "GET", null, "", S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                    if (RetBool == true)
                    {
                        MyXmlDocument = new System.Xml.XmlDocument();
                        MyXmlDocument.LoadXml(ResponseString);

                        MyXmlNamespaceManager = new System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable);
                        MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/");

                        MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListMultipartUploadsResult/amz:Upload", MyXmlNamespaceManager);

                        foreach (System.Xml.XmlNode ItemXmlNode in MyXmlNodeList)
                        {
                            MyXmlNode = ItemXmlNode.SelectSingleNode("amz:Key", MyXmlNamespaceManager);

                            if (MyXmlNode.InnerText == KeyName)
                            {
                                MyXmlNode = ItemXmlNode.SelectSingleNode("amz:UploadId", MyXmlNamespaceManager);
                                UploadId = MyXmlNode.InnerText;
                                break;
                            }
                        }
                    }
                    else
                    {
                        ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                        ReturnValue = false;
                    }
                }


                if (ReturnValue == true)
                {
                    if (UploadId == "")
                    {
                        //init a new multipart upload
                        //Initiate Multipart Upload: http://docs.amazonwebservices.com/AmazonS3/latest/API/mpUploadInitiate.html

                        RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, KeyName, "?uploads");
                        RequestMethod = "POST";

                        RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, UserRequestHeaders, "", S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                        if (RetBool == true)
                        {
                            MyXmlDocument = new System.Xml.XmlDocument();
                            MyXmlDocument.LoadXml(ResponseString);

                            MyXmlNamespaceManager = new System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable);
                            MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/");

                            MyXmlNode = MyXmlDocument.SelectSingleNode("amz:InitiateMultipartUploadResult/amz:UploadId", MyXmlNamespaceManager);

                            UploadId = MyXmlNode.InnerText;
                        }
                        else
                        {
                            ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                            ReturnValue = false;
                        }
                    }
                }


                if (ReturnValue == true)
                {
                    //Check if parts exist on S3.

                    System.Collections.Hashtable PartsHashtable = new System.Collections.Hashtable();
                    String NextPartNumberMarker = "";
                    Boolean IsTruncated = false;
                    int PartNumber;
                    String ETag;

                    do
                    {
                        //List Parts: http://docs.amazonwebservices.com/AmazonS3/latest/API/mpUploadListParts.html

                        QueryString = "?uploadId=" + System.Uri.EscapeDataString(UploadId) + "&max-parts=1000";
                        if (NextPartNumberMarker != "")
                        {
                            QueryString += "&part-number-marker=" + NextPartNumberMarker;
                        }
                        RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, KeyName, QueryString);
                        RequestMethod = "GET";

                        RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, null, "", S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                        if (RetBool == true)
                        {
                            MyXmlDocument = new System.Xml.XmlDocument();
                            MyXmlDocument.LoadXml(ResponseString);

                            MyXmlNamespaceManager = new System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable);
                            MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/");

                            MyXmlNodeList = MyXmlDocument.SelectNodes("amz:ListPartsResult/amz:Part", MyXmlNamespaceManager);
                            foreach (System.Xml.XmlNode ItemXmlNode in MyXmlNodeList)
                            {
                                MyXmlNode = ItemXmlNode.SelectSingleNode("amz:PartNumber", MyXmlNamespaceManager);
                                PartNumber = Convert.ToInt32(MyXmlNode.InnerText);

                                MyXmlNode = ItemXmlNode.SelectSingleNode("amz:ETag", MyXmlNamespaceManager);
                                ETag = MyXmlNode.InnerText;
                                ETag = ETag.Replace("\"", "");

                                PartsHashtable.Add(PartNumber, ETag);
                            }

                            MyXmlNode = MyXmlDocument.SelectSingleNode("amz:ListPartsResult/amz:IsTruncated", MyXmlNamespaceManager);
                            if (MyXmlNode == null)
                            {
                                IsTruncated = false;
                            }
                            else
                            {
                                if (MyXmlNode.InnerText == "true")
                                {
                                    IsTruncated = true;

                                    MyXmlNode = MyXmlDocument.SelectSingleNode("amz:ListPartsResult/amz:NextPartNumberMarker", MyXmlNamespaceManager);
                                    NextPartNumberMarker = MyXmlNode.InnerText;
                                }
                                else
                                {
                                    IsTruncated = false;
                                }
                            }
                        }
                        else
                        {
                            ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                            ReturnValue = false;
                            break;
                        }

                    } while (IsTruncated == true);


                    if (ReturnValue == true)
                    {
                        long OffsetStart = 0;
                        long OffsetEnd = 0;

                        String RemoteMD5;
                        String RemoteETag;
                        String PostXML = "";

                        //Build the XML uses to complete the multipart upload.
                        PostXML += "<CompleteMultipartUpload>" + Environment.NewLine;

                        do
                        {
                            PartCounter += 1;

                            //Get the byte range of the part to upload.
                            OffsetStart = (MultipartPartSizeBytes * PartCounter) - MultipartPartSizeBytes;
                            OffsetEnd = (MultipartPartSizeBytes * PartCounter) - 1;

                            if (OffsetEnd >= MyFileInfo.Length)
                            {
                                OffsetEnd = MyFileInfo.Length - 1;
                            }

                            //Check if the part has already been uploaded.
                            if (PartsHashtable.ContainsKey(PartCounter) == true)
                            {
                                //The part exists on S3, do not upload it.  Check if the MD5 is the same.

                                LocalMD5 = MyCalculateHash.CalculateMD5FromFile(LocalFile, OffsetStart, OffsetEnd);

                                RemoteETag = PartsHashtable[PartCounter].ToString();
                                RemoteMD5 = MyS3Helper.ConvertETagToMD5(RemoteETag);

                                if (LocalMD5 == RemoteMD5)
                                {
                                    //Build the XML uses to complete the multipart upload.
                                    PostXML += "  <Part>" + Environment.NewLine;
                                    PostXML += "    <PartNumber>" + PartCounter + "</PartNumber>" + Environment.NewLine;
                                    PostXML += "    <ETag>" + RemoteETag + "</ETag>" + Environment.NewLine;
                                    PostXML += "  </Part>" + Environment.NewLine;
                                }
                                else
                                {
                                    //The user needs to cancel the existing multipart upload.
                                    ResponseErrorMessage = "Error, the MD5 of a local part does not match it's part on S3.  Abort the existing multipart upload.  PartNumber=" + PartCounter + "  LocalMD5=" + LocalMD5 + "  RemoteMD5=" + RemoteMD5;
                                    ReturnValue = false;
                                    break;
                                }
                            }
                            else
                            {
                                //Upload Part: http://docs.amazonwebservices.com/AmazonS3/latest/API/mpUploadUploadPart.html

                                QueryString = "?uploadId=" + System.Uri.EscapeDataString(UploadId) + "&partNumber=" + PartCounter;
                                RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, KeyName, QueryString);
                                RequestMethod = "PUT";

                                ExtraRequestHeaders = new Dictionary<String, String>();

                                //Calculate the MD5 hash of the part that will be uploaded.
                                LocalMD5 = MyCalculateHash.CalculateMD5FromFile(LocalFile, OffsetStart, OffsetEnd);
                                ExtraRequestHeaders.Add("Content-MD5", LocalMD5);

                                RetBool = UploadFileToS3(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, ExtraRequestHeaders, LocalFile, OffsetStart, OffsetEnd, S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                                if (RetBool == true)
                                {
                                    //Build the XML uses to complete the multipart upload.
                                    PostXML += "  <Part>" + Environment.NewLine;
                                    PostXML += "    <PartNumber>" + PartCounter + "</PartNumber>" + Environment.NewLine;
                                    PostXML += "    <ETag>" + MyS3Helper.ConvertMD5ToETag(LocalMD5) + "</ETag>" + Environment.NewLine;
                                    PostXML += "  </Part>" + Environment.NewLine;
                                }
                                else
                                {
                                    ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                                    ReturnValue = false;
                                    break;
                                }
                            }

                        } while (OffsetEnd < MyFileInfo.Length - 1);


                        if (ReturnValue == true)
                        {
                            //Complete the multipart upload.

                            PostXML += "</CompleteMultipartUpload>" + Environment.NewLine;

                            QueryString = "?uploadId=" + System.Uri.EscapeDataString(UploadId);
                            RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, KeyName, QueryString);
                            RequestMethod = "POST";

                            RetBool = MakeS3Request(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, null, PostXML, S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                            if (RetBool == true)
                            {
                                MyXmlDocument = new System.Xml.XmlDocument();
                                MyXmlDocument.LoadXml(ResponseString);

                                MyXmlNamespaceManager = new System.Xml.XmlNamespaceManager(MyXmlDocument.NameTable);
                                MyXmlNamespaceManager.AddNamespace("amz", "http://s3.amazonaws.com/doc/2006-03-01/");

                                if (MyXmlDocument.SelectSingleNode("amz:Error", MyXmlNamespaceManager) == null)
                                {
                                    //Upload successful.
                                }
                                else
                                {
                                    ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                                    ReturnValue = false;
                                }
                            }
                            else
                            {
                                ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                                ReturnValue = false;
                            }
                        }
                    }
                }
            }
            else
            {
                //The file is too small for a multipart upload.  Use a regular upload.

                RequestURL = MyREST.BuildS3RequestURL(UseSSL, RequestEndpoint, BucketName, KeyName, "");
                RequestMethod = "PUT";

                ExtraRequestHeaders = new Dictionary<String, String>();

                //Add the user's headers.
                foreach (KeyValuePair<String, String> MyKeyValuePair in UserRequestHeaders)
                {
                    ExtraRequestHeaders.Add(MyKeyValuePair.Key, MyKeyValuePair.Value);
                }

                //Calculate the MD5 hash of the file.
                if (LocalETag == "")
                {
                    LocalMD5 = MyCalculateHash.CalculateMD5FromFile(LocalFile);
                }
                else
                {
                    LocalMD5 = MyS3Helper.ConvertETagToMD5(LocalETag);
                }
                ExtraRequestHeaders.Add("Content-MD5", LocalMD5);

                RetBool = UploadFileToS3(AWSAccessKeyId, AWSSecretAccessKey, RequestURL, RequestMethod, ExtraRequestHeaders, LocalFile, -1, -1, S3ErrorRetries, ref ErrorNumber, ref ErrorDescription, ref LogData, ref RequestHeaders, ref ResponseStatusCode, ref ResponseStatusDescription, ref ResponseHeaders, ref ResponseString);

                if (RetBool == true)
                {
                    //Upload successful
                }
                else
                {
                    ResponseErrorMessage = FormatErrorData(RequestURL, RequestMethod, RequestHeaders, ResponseStatusCode, ResponseStatusDescription, ResponseHeaders, ResponseString, ErrorNumber, ErrorDescription);
                    ReturnValue = false;
                }
            }

            return ReturnValue;
        }


        private static Boolean MakeS3Request(String AWSAccessKeyId, String AWSSecretAccessKey, String RequestURL, String RequestMethod, System.Collections.Generic.Dictionary<String, String> UserInputRequestHeaders, String SendData, int RetryTimes, ref int ErrorNumber, ref String ErrorDescription, ref String LogData, ref System.Collections.Generic.Dictionary<String, String> RequestHeaders, ref int ResponseStatusCode, ref String ResponseStatusDescription, ref System.Collections.Generic.Dictionary<String, String> ResponseHeaders, ref String ResponseStringFormatted)
        {
            SprightlySoftAWS.REST MyREST = new SprightlySoftAWS.REST();
            System.Net.IWebProxy MyIWebProxy;
            MyIWebProxy = System.Net.WebRequest.DefaultWebProxy;
            MyIWebProxy.Credentials = System.Net.CredentialCache.DefaultCredentials;
            MyREST.Proxy = MyIWebProxy;

            Boolean RetBool = true;
            LogData = "";

            Dictionary<String, String> FullInputRequestHeaders;
            String AuthorizationValue;

            for (int i = 0; i <= RetryTimes; i++)
            {
                FullInputRequestHeaders = new Dictionary<String, String>();

                if (UserInputRequestHeaders != null)
                {
                    foreach (KeyValuePair<String, String> MyKeyValuePair in UserInputRequestHeaders)
                    {
                        FullInputRequestHeaders.Add(MyKeyValuePair.Key, MyKeyValuePair.Value);
                    }
                }

                //Set the date for each request.
                FullInputRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));

                AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, FullInputRequestHeaders, AWSAccessKeyId, AWSSecretAccessKey);
                FullInputRequestHeaders.Add("Authorization", AuthorizationValue);

                RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, FullInputRequestHeaders, SendData);

                //Set the return values.
                ErrorNumber = MyREST.ErrorNumber;
                ErrorDescription = MyREST.ErrorDescription;
                LogData += MyREST.LogData + Environment.NewLine;
                ResponseStatusCode = MyREST.ResponseStatusCode;
                ResponseStatusDescription = MyREST.ResponseStatusDescription;
                ResponseHeaders = MyREST.ResponseHeaders;
                ResponseStringFormatted = MyREST.ResponseStringFormatted;

                if (RetBool == true)
                {
                    break;
                }
                else
                {
                    if (MyREST.ResponseStatusCode == 500 || MyREST.ResponseStatusCode == 503)
                    {
                        //A Service Unavailable response was returned.  Wait and retry.
                        System.Threading.Thread.Sleep(1000 * i * i);
                    }
                    else if (MyREST.ResponseStatusCode == 0 && MyREST.ErrorNumber == 1003)
                    {
                        //Getting the response failed.  This may be a network disconnection.  Wait and retry.
                        System.Threading.Thread.Sleep(1000 * i * i);
                    }
                    else
                    {
                        //An error occured but retrying would not solve the problem.
                        break;
                    }
                }
            }

            return RetBool;
        }


        private static Boolean UploadFileToS3(String AWSAccessKeyId, String AWSSecretAccessKey, String RequestURL, String RequestMethod, System.Collections.Generic.Dictionary<String, String> UserInputRequestHeaders, String LocalFile, long OffsetStart, long OffsetEnd, int RetryTimes, ref int ErrorNumber, ref String ErrorDescription, ref String LogData, ref System.Collections.Generic.Dictionary<String, String> RequestHeaders, ref int ResponseStatusCode, ref String ResponseStatusDescription, ref System.Collections.Generic.Dictionary<String, String> ResponseHeaders, ref String ResponseStringFormatted)
        {
            Boolean RetBool = true;
            LogData = "";

            Dictionary<String, String> FullInputRequestHeaders;
            String AuthorizationValue;

            for (int i = 0; i <= RetryTimes; i++)
            {
                FullInputRequestHeaders = new Dictionary<String, String>();

                if (UserInputRequestHeaders != null)
                {
                    foreach (KeyValuePair<String, String> MyKeyValuePair in UserInputRequestHeaders)
                    {
                        FullInputRequestHeaders.Add(MyKeyValuePair.Key, MyKeyValuePair.Value);
                    }
                }

                FullInputRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"));
                AuthorizationValue = MyUpload.GetS3AuthorizationValue(RequestURL, RequestMethod, FullInputRequestHeaders, AWSAccessKeyId, AWSSecretAccessKey);
                FullInputRequestHeaders.Add("Authorization", AuthorizationValue);

                RetBool = MyUpload.UploadFile(RequestURL, RequestMethod, FullInputRequestHeaders, LocalFile, OffsetStart, OffsetEnd);

                //Set the return values.
                ErrorNumber = MyUpload.ErrorNumber;
                ErrorDescription = MyUpload.ErrorDescription;
                LogData += MyUpload.LogData + Environment.NewLine;
                RequestHeaders = MyUpload.RequestHeaders;
                ResponseStatusCode = MyUpload.ResponseStatusCode;
                ResponseStatusDescription = MyUpload.ResponseStatusDescription;
                ResponseHeaders = MyUpload.ResponseHeaders;
                ResponseStringFormatted = MyUpload.ResponseStringFormatted;

                if (RetBool == true)
                {
                    break;
                }
                else
                {
                    if (MyUpload.ResponseStatusCode == 500 || MyUpload.ResponseStatusCode == 503)
                    {
                        //A Service Unavailable response was returned.  Wait and retry.
                        System.Threading.Thread.Sleep(1000 * i * i);
                    }
                    else if (MyUpload.ResponseStatusCode == 0 && MyUpload.ErrorNumber == 1003)
                    {
                        //Getting the response failed.  This may be a network disconnection.  Wait and retry.
                        System.Threading.Thread.Sleep(1000 * i * i);
                    }
                    else if (MyUpload.ErrorNumber == 1004)
                    {
                        //Opening the request stream failed.  This may be a network disconnection.  Wait and retry.
                        System.Threading.Thread.Sleep(1000 * i * i);
                    }
                    else
                    {
                        //An error occured but retrying would not solve the problem.
                        break;
                    }
                }
            }

            return RetBool;
        }


        private static void MyUpload_ProgressChangedEvent()
        {
            if (ShowTransferProgress == true)
            {
                if (MyUpload.BytesTotal > 0)
                {
                    if (DateTime.Now >= ProgressUpdateShown.AddMilliseconds(500))  //only show update once every 0.5 sec
                    {
                        long TotalTransfered;
                        TotalTransfered = (MultipartPartSizeBytes * (PartCounter - 1)) + MyUpload.BytesTransfered;

                        SprightlySoftAWS.S3.Helper MyS3Helper = new SprightlySoftAWS.S3.Helper();
                        Double PercentPart = ((Double)MyUpload.BytesTransfered / (Double)MyUpload.BytesTotal) * 100;
                        Double PercentTotal = ((Double)TotalTransfered / (Double)FileLength) * 100;

                        Console.Write(PercentPart.ToString("0.00") + "% part " + PartCounter + " of " + Math.Ceiling((Double)FileLength / (Double)MultipartPartSizeBytes) + "   " + MyS3Helper.FormatByteSize(MyUpload.BytesTransfered) + " / " + MyS3Helper.FormatByteSize(MyUpload.BytesTotal) + "   " + MyUpload.CurrentKBpsSpeed.ToString("0.00") + " KB/s   " + PercentTotal.ToString("0.00") + "% of total     \r");

                        ProgressUpdateShown = DateTime.Now;
                    }
                }
            }
        }


        private static void MyUpload_StateChangedEvent()
        {
            if (ShowTransferProgress == true)
            {
                if (MyUpload.State == 1)
                {
                    //Reset the ProgressUpdateShown value so the progress is shown when the first ProgressChangedEvent occurs.
                    ProgressUpdateShown = DateTime.Now.AddMilliseconds(-500);
                }
                else if (MyUpload.State == 0)
                {
                    //Clear the progress line on complete.
                    Console.Write("                                                                            \r");
                }
            }
        }


        private static String FormatErrorData(String RequestURL, String RequestMethod, Dictionary<String, String> RequestHeaders, int ResponseStatusCode, String ResponseStatusDescription, Dictionary<String, String> ResponseHeaders, String ResponseString, int ErrorNumber, String ErrorDescription)
        {
            String ReturnString = "";
            ReturnString += "An error occurred." + Environment.NewLine;

            if (OutputLevel == 1)
            {
                if (ResponseString == "")
                {
                    ReturnString += "Component ErrorDescription: " + ErrorDescription + Environment.NewLine;
                    ReturnString += "Component ErrorNumber: " + ErrorNumber + Environment.NewLine;
                    ReturnString += "Response Status Code: " + ResponseStatusCode + Environment.NewLine;
                    ReturnString += "Response Status Description: " + ResponseStatusDescription;
                }
                else
                {
                    try
                    {
                        System.Xml.XmlDocument XmlDoc = new System.Xml.XmlDocument();
                        XmlDoc.LoadXml(ResponseString);

                        System.Xml.XmlNode XmlNode;
                        XmlNode = XmlDoc.SelectSingleNode("/Error/Message");

                        if (XmlNode == null)
                        {
                            ReturnString += "Response XML: " + Environment.NewLine + ResponseString;
                        }
                        else
                        {
                            ReturnString += "   Amazon Error Message: " + XmlNode.InnerText;
                        }
                    }
                    catch (Exception e)
                    {
                        ReturnString += "Response String: " + Environment.NewLine + ResponseString;
                    }
                }
            }
            else
            {
                if (ErrorNumber != 0)
                {
                    ReturnString += "Component ErrorDescription: " + ErrorDescription + Environment.NewLine;
                    ReturnString += "Component ErrorNumber: " + ErrorNumber + Environment.NewLine;
                    ReturnString += Environment.NewLine;
                }

                if (ResponseStatusCode != 0)
                {
                    ReturnString += "Request URL: " + RequestURL + Environment.NewLine;
                    ReturnString += "Request Method: " + RequestMethod + Environment.NewLine;

                    foreach (KeyValuePair<String, String> MyHeader in RequestHeaders)
                    {
                        ReturnString += "Request Header: " + MyHeader.Key + ":" + MyHeader.Value + Environment.NewLine;
                    }

                    ReturnString += Environment.NewLine;
                    ReturnString += "Response Status Code: " + ResponseStatusCode + Environment.NewLine;
                    ReturnString += "Response Status Description: " + ResponseStatusDescription + Environment.NewLine;

                    foreach (KeyValuePair<String, String> MyHeader in ResponseHeaders)
                    {
                        ReturnString += "Response Header: " + MyHeader.Key + ":" + MyHeader.Value + Environment.NewLine;
                    }

                    if (ResponseString != "")
                    {
                        ReturnString += Environment.NewLine;

                        try
                        {
                            System.Xml.XmlDocument XmlDoc = new System.Xml.XmlDocument();
                            XmlDoc.LoadXml(ResponseString);

                            ReturnString += "Response XML: " + Environment.NewLine + ResponseString + Environment.NewLine;
                        }
                        catch (Exception e)
                        {
                            ReturnString += "Response String: " + Environment.NewLine + ResponseString + Environment.NewLine;
                        }
                    }
                }
            }

            return ReturnString;
        }


        private static String GetValueFromConfigOrArgument(string[] args, String ArgumentName)
        {
            String CurrentValue;
            CurrentValue = System.Configuration.ConfigurationManager.AppSettings[ArgumentName];

            ArgumentName = ArgumentName.ToLower();
            String CurrentArg;

            for (int i = 0; i < args.Length; i++)
            {
                CurrentArg = args[i].ToLower();

                if (CurrentArg.StartsWith("-" + ArgumentName) | CurrentArg.StartsWith("/" + ArgumentName) | CurrentArg.StartsWith("–" + ArgumentName))
                {
                    try
                    {
                        CurrentValue = args[i + 1];
                    }
                    catch (Exception ex)
                    {
                        WriteToOutput("Error reading command line value.  ArgumentName=" + ArgumentName + "   ex.Message=" + ex.Message);
                    }

                    break;
                }
            }

            if (CurrentValue == null)
            {
                CurrentValue = "";
            }

            return CurrentValue;
        }


        private static Dictionary<String, String> BuildDictionaryFromString(String InputString)
        {
            Dictionary<String, String> MyDictionary = new Dictionary<String, String>();

            if (InputString.Contains(":") == true)
            {
                try
                {
                    //Split multiple headers with the comma character.
                    String[] HeadersArray;
                    HeadersArray = InputString.Split(new Char[] { '|' });

                    String[] PairArray;
                    foreach (String ArrayValue in HeadersArray)
                    {
                        //Split the name and value with the colon character.
                        PairArray = ArrayValue.Split(new Char[] { ':' });

                        MyDictionary.Add(PairArray[0], PairArray[1]);
                    }
                }
                catch (Exception ex)
                {
                    WriteToOutput("Error parsing the UploadHeaders.  ex.Message=" + ex.Message);
                }
            }

            return MyDictionary;
        }


        private static void WriteToOutput(String OutputLine, int LogOutputLevel = 1, Boolean IsError = false)
        {
            if (LogOutputLevel <= OutputLevel)
            {
                System.Diagnostics.Debug.Print(OutputLine);

                if (IsError == true)
                {
                    System.IO.TextWriter MyTextWriterError = Console.Error;
                    MyTextWriterError.WriteLine(OutputLine);
                }
                else
                {
                    Console.WriteLine(OutputLine);
                }
            }
        }


        static void MyUnhandledException(object sender, UnhandledExceptionEventArgs args)
        {
            Exception e = (Exception)args.ExceptionObject;

            System.IO.TextWriter MyTextWriterError = Console.Error;

            MyTextWriterError.WriteLine("Error in application.");
            MyTextWriterError.WriteLine("Message=" + e.Message);
            MyTextWriterError.WriteLine("StackTrace=" + e.StackTrace);

            Environment.ExitCode = -1;
        }

    }
}
