﻿Imports System.Windows.Forms

Public Class FormUploadEstimate

    Dim WithEvents MyCalculateHash As SprightlySoftAWS.S3.CalculateHash
    Dim WithEvents MyUpload As SprightlySoftAWS.S3.UploadEstimate
    Dim WithEvents UploadBackgroundWorker As System.ComponentModel.BackgroundWorker
    Dim WithEvents CalculateHashBackgroundWorker As System.ComponentModel.BackgroundWorker
    Dim WithEvents ETagBackgroundWorker As System.ComponentModel.BackgroundWorker
    Dim TimerUpdateStatus As System.Timers.Timer

    Private Sub FormUpload_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        MyCalculateHash = New SprightlySoftAWS.S3.CalculateHash
        MyUpload = New SprightlySoftAWS.S3.UploadEstimate

        CalculateHashBackgroundWorker = New System.ComponentModel.BackgroundWorker
        UploadBackgroundWorker = New System.ComponentModel.BackgroundWorker
        ETagBackgroundWorker = New System.ComponentModel.BackgroundWorker

        'use a timer to show the current speed and time remaining
        TimerUpdateStatus = New System.Timers.Timer
        AddHandler TimerUpdateStatus.Elapsed, AddressOf TimerUpdateStatus_Tick
        TimerUpdateStatus.Interval = 500
    End Sub

    Private Sub EnableDisableStart()
        TextBoxUploadFileName.Enabled = False
        TextBoxUploadBucketName.Enabled = False
        TextBoxUploadKeyName.Enabled = False
        TextBoxUploadContentType.Enabled = False
        CheckBoxUploadMakePublic.Enabled = False
        ButtonUploadBrowse.Enabled = False
        ButtonUploadFile.Enabled = False
        ButtonAbort.Enabled = True

        ToolStripStatusLabelAction.Text = ""
        ToolStripProgressBar.Value = 0
        ToolStripStatusLabelBytes.Text = ""
        ToolStripStatusLabelSpeed.Text = ""
        ToolStripStatusLabelElapsed.Text = ""
        ToolStripStatusLabelRemaining.Text = ""

        TimerUpdateStatus.Enabled = True
    End Sub

    Private Sub EnableDisableEnd()
        TextBoxUploadFileName.Enabled = True
        TextBoxUploadBucketName.Enabled = True
        TextBoxUploadKeyName.Enabled = True
        TextBoxUploadContentType.Enabled = True
        CheckBoxUploadMakePublic.Enabled = True
        ButtonUploadBrowse.Enabled = True
        ButtonUploadFile.Enabled = True
        ButtonAbort.Enabled = False

        ToolStripStatusLabelAction.Visible = False
        ToolStripProgressBar.Visible = False
        ToolStripStatusLabelBytes.Visible = False
        ToolStripStatusLabelSpeed.Visible = False
        ToolStripStatusLabelElapsed.Visible = False
        ToolStripStatusLabelRemaining.Visible = False

        TimerUpdateStatus.Enabled = False
    End Sub

    Private Sub ButtonUploadBrowse_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonUploadBrowse.Click
        Dim OpenFileDialog As New System.Windows.Forms.OpenFileDialog
        OpenFileDialog.Multiselect = False
        OpenFileDialog.FileName = ""
        OpenFileDialog.ShowDialog()

        If OpenFileDialog.FileName <> "" AndAlso System.IO.File.Exists(OpenFileDialog.FileName) = True Then

            TextBoxUploadFileName.Text = OpenFileDialog.FileName
            TextBoxUploadKeyName.Text = System.IO.Path.GetFileName(OpenFileDialog.FileName)

            'set the content type from the file's extension
            'the S3 Helper class contains a function that returns a dictionary of extensions and associated content types
            Dim MyHelper As New SprightlySoftAWS.S3.Helper
            Dim ContentTypesDictionary As Dictionary(Of String, String)
            ContentTypesDictionary = MyHelper.GetContentTypesDictionary

            Dim MyExtension As String
            MyExtension = LCase(System.IO.Path.GetExtension(OpenFileDialog.FileName))

            If ContentTypesDictionary.ContainsKey(MyExtension) = True Then
                TextBoxUploadContentType.Text = ContentTypesDictionary(MyExtension)
            Else
                TextBoxUploadContentType.Text = ""
            End If

        End If
    End Sub

    Private Sub ButtonUploadFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonUploadFile.Click
        EnableDisableStart()

        If System.IO.File.Exists(TextBoxUploadFileName.Text) = True Then

            ToolStripStatusLabelAction.Text = "Checking if file exists...."
            ToolStripStatusLabelAction.Visible = True
            Application.DoEvents()

            'check if the file exists on amazon
            Dim MyREST As New SprightlySoftAWS.REST

            Dim RequestURL As String
            RequestURL = MyREST.BuildS3RequestURL(True, "s3.amazonaws.com", TextBoxUploadBucketName.Text, TextBoxUploadKeyName.Text, "")

            Dim RequestMethod As String = "HEAD"

            Dim ExtraRequestHeaders As New Dictionary(Of String, String)
            ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"))

            Dim AuthorizationValue As String
            AuthorizationValue = MyREST.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, FormMain.TextBoxAWSAccessKeyId.Text, FormMain.TextBoxAWSSecretAccessKey.Text)
            ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

            Dim RetBool As Boolean
            RetBool = MyREST.MakeRequest(RequestURL, RequestMethod, ExtraRequestHeaders, "")

            ToolStripStatusLabelAction.Text = ""

            If MyREST.ResponseStatusCode = 200 Then
                'the file exists.  Show the conflict form
                FormConflictUpload.TextBoxRemoteBucketName.Text = TextBoxUploadBucketName.Text
                FormConflictUpload.TextBoxRemoteKeyName.Text = TextBoxUploadKeyName.Text
                FormConflictUpload.TextBoxRemoteSize.Text = FormatNumber(MyREST.ResponseHeaders("Content-Length"), 0) & " bytes"
                FormConflictUpload.TextBoxRemoteAmazonDate.Text = CDate(MyREST.ResponseHeaders("Last-Modified"))
                FormConflictUpload.TextBoxRemoteETag.Text = Replace(MyREST.ResponseHeaders("ETag"), """", "")

                Dim LocalFileInfo As New System.IO.FileInfo(TextBoxUploadFileName.Text)
                FormConflictUpload.TextBoxLocalFileName.Text = TextBoxUploadFileName.Text
                FormConflictUpload.TextBoxLocalSize.Text = FormatNumber(LocalFileInfo.Length, 0) & " bytes"
                FormConflictUpload.TextBoxLocalDate.Text = LocalFileInfo.LastWriteTime

                'kick off a process to calculate the etag.  A large file may take a long time to calculate.
                FormConflictUpload.TextBoxLocalETag.Text = "calculating...."
                Dim ETagHashTable As New Hashtable
                ETagHashTable.Add("LocalFileName", TextBoxUploadFileName.Text)
                ETagBackgroundWorker.RunWorkerAsync(ETagHashTable)

                'show the conflict form
                Dim DialogResult As System.Windows.Forms.DialogResult
                DialogResult = FormConflictUpload.ShowDialog(Me)

                If DialogResult = Windows.Forms.DialogResult.Yes Then
                    'the use agreed to overwrite the file, do nothing
                Else
                    EnableDisableEnd()
                    MessageBox.Show("Upload skipped.", "Skipped", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    Exit Sub
                End If

            End If


            'abort the calculation of the etag if it is taking place
            If MyCalculateHash.State = 1 Then
                MyCalculateHash.Abort()
                'wait until the abort sequence is complete
                Do While MyCalculateHash.State = 1
                    System.Threading.Thread.Sleep(100)
                Loop
            End If

            'Start by calculating the MD5 hash of the file.  This will be sent to Amazon in the 
            'upload to make sure it arrives correctly.
            ToolStripStatusLabelAction.Text = "Calculating MD5 hash...."
            Application.DoEvents()

            'Run the hash calculation in a BackgroundWorker process.  Calculating the hash of a
            'large file will take a while.  Running the process in a BackgroundWorker will prevent
            'the form from locking up.

            'Use a hash table to pass parameters to the function in the BackgroundWorker.
            Dim CalculateHashHashTable As New Hashtable
            CalculateHashHashTable.Add("LocalFileName", TextBoxUploadFileName.Text)

            CalculateHashBackgroundWorker.RunWorkerAsync(CalculateHashHashTable)


        Else
            EnableDisableEnd()
            MessageBox.Show("The local file does not exist.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If

    End Sub

    Private Sub CalculateHashBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles CalculateHashBackgroundWorker.DoWork
        'Call the CalculateMD5FromFile function and set the result.  When the function is complete
        'the RunWorkerCompleted event will fire.  Use the LocalFileName value from the passed hash table.
        e.Result = MyCalculateHash.CalculateMD5FromFile(e.Argument("LocalFileName"))
    End Sub

    Private Sub CalculateHashBackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles CalculateHashBackgroundWorker.RunWorkerCompleted

        'If the CalculateMD5FromFile function was successful upload the file.
        If MyCalculateHash.ErrorNumber = 0 Then

            ToolStripStatusLabelAction.Visible = False
            ToolStripProgressBar.Visible = True
            ToolStripStatusLabelBytes.Visible = True
            ToolStripStatusLabelSpeed.Visible = True
            ToolStripStatusLabelElapsed.Visible = True
            ToolStripStatusLabelRemaining.Visible = True

            Application.DoEvents()

            'Set the extra request headers to send with the upload
            Dim ExtraRequestHeaders As New Dictionary(Of String, String)

            If TextBoxUploadContentType.Text <> "" Then
                ExtraRequestHeaders.Add("Content-Type", TextBoxUploadContentType.Text)
            End If

            If CheckBoxUploadMakePublic.Checked = True Then
                ExtraRequestHeaders.Add("x-amz-acl", "public-read")
            End If

            'Use the MD5 hash that was calculated previously.
            ExtraRequestHeaders.Add("Content-MD5", e.Result)

            Dim RequestURL As String
            RequestURL = MyUpload.BuildS3RequestURL(True, "s3.amazonaws.com", TextBoxUploadBucketName.Text, TextBoxUploadKeyName.Text, "")

            Dim RequestMethod As String = "PUT"

            ExtraRequestHeaders.Add("x-amz-date", DateTime.UtcNow.ToString("r"))

            Dim AuthorizationValue As String
            AuthorizationValue = MyUpload.GetS3AuthorizationValue(RequestURL, RequestMethod, ExtraRequestHeaders, FormMain.TextBoxAWSAccessKeyId.Text, FormMain.TextBoxAWSSecretAccessKey.Text)
            ExtraRequestHeaders.Add("Authorization", AuthorizationValue)

            'Create a hash table of of parameters to sent to the upload function.
            Dim UploadHashTable As New Hashtable
            UploadHashTable.Add("RequestURL", RequestURL)
            UploadHashTable.Add("RequestMethod", RequestMethod)
            UploadHashTable.Add("ExtraRequestHeaders", ExtraRequestHeaders)
            UploadHashTable.Add("LocalFileName", TextBoxUploadFileName.Text)

            'Run the UploadFile call in a BackgroundWorker to prevent the Window from freezing.
            UploadBackgroundWorker.RunWorkerAsync(UploadHashTable)

        Else
            EnableDisableEnd()
            MessageBox.Show(MyCalculateHash.ErrorDescription, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If

    End Sub

    Private Sub UploadBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles UploadBackgroundWorker.DoWork
        'Run the UploadFile call.
        e.Result = MyUpload.UploadFile(e.Argument("RequestURL"), e.Argument("RequestMethod"), e.Argument("ExtraRequestHeaders"), e.Argument("LocalFileName"))
    End Sub

    Private Sub UploadBackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles UploadBackgroundWorker.RunWorkerCompleted

        Debug.Print("")
        Debug.Print(MyUpload.LogData)
        Debug.Print("")

        EnableDisableEnd()

        If e.Result = True Then
            MessageBox.Show("Upload complete.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Else
            'Show the error message.
            Dim ResponseMessage As String

            If MyUpload.ResponseString = "" Then
                ResponseMessage = MyUpload.ErrorDescription
            Else
                Dim XmlDoc As New System.Xml.XmlDocument
                XmlDoc.LoadXml(MyUpload.ResponseString)

                Dim XmlNode As System.Xml.XmlNode
                XmlNode = XmlDoc.SelectSingleNode("/Error/Message")

                ResponseMessage = XmlNode.InnerText
            End If

            MessageBox.Show(ResponseMessage, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)
        End If

    End Sub

    Private Sub ButtonAbort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ButtonAbort.Click
        'Abort the CalculateHash call or Upload call depending on what is taking place.
        If MyCalculateHash.State = 1 Then
            MyCalculateHash.Abort()
        ElseIf MyUpload.State = 1 Then
            MyUpload.Abort()
        End If
    End Sub

    Private Sub MyUpload_ProgressChangedEvent() Handles MyUpload.ProgressChangedEvent
        If Me.InvokeRequired = True Then
            Me.Invoke(New MethodInvoker(AddressOf MyUpload_ProgressChangedEvent))
        Else
            'Update the progress bar when the ProgressChangedEvent is fired.
            If MyUpload.BytesTotal > 0 Then
                ToolStripProgressBar.Value = (MyUpload.BytesTransfered / MyUpload.BytesTotal) * 100

                Dim MyHelper As New SprightlySoftAWS.S3.Helper
                ToolStripStatusLabelBytes.Text = MyHelper.FormatByteSize(MyUpload.BytesTransfered) & " / " & MyHelper.FormatByteSize(MyUpload.BytesTotal)
            End If
        End If
    End Sub

    Private Sub TimerUpdateStatus_Tick()

        If Me.InvokeRequired = True Then
            Me.Invoke(New MethodInvoker(AddressOf TimerUpdateStatus_Tick))
        Else
            'get the current speed, time elapsed and time remaining from the UploadEstimate class
            ToolStripStatusLabelSpeed.Text = FormatNumber(MyUpload.CurrentKBpsSpeed, 2) & " KB/s"

            Dim ElapsedTimeSpan As TimeSpan
            ElapsedTimeSpan = New TimeSpan(0, 0, MyUpload.SecondsElapsed)
            ToolStripStatusLabelElapsed.Text = ElapsedTimeSpan.ToString & " elapsed"

            Dim RemainingTimeSpan As TimeSpan
            RemainingTimeSpan = New TimeSpan(0, 0, MyUpload.SecondsRemaining)
            ToolStripStatusLabelRemaining.Text = RemainingTimeSpan.ToString & " remaining"
        End If

    End Sub

    Private Sub ETagBackgroundWorker_DoWork(ByVal sender As System.Object, ByVal e As System.ComponentModel.DoWorkEventArgs) Handles ETagBackgroundWorker.DoWork
        e.Result = MyCalculateHash.CalculateETagFromFile(e.Argument("LocalFileName"))
    End Sub

    Private Sub ETagBackgroundWorker_RunWorkerCompleted(ByVal sender As Object, ByVal e As System.ComponentModel.RunWorkerCompletedEventArgs) Handles ETagBackgroundWorker.RunWorkerCompleted
        'show the etag in the conflict form
        FormConflictUpload.TextBoxLocalETag.Text = e.Result
    End Sub

End Class
